﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO.Ports;
using System.Linq;
using System.Management;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;



namespace WindowsFormsApp1
{
    public partial class Form1 : Form
    {
        // Define a class for the individual lines
        private class Linea
        {
            public int angle { get; set; }
            public int alpha { get; set; }
            public Color color { get; set; }
            public int distancia { get; set; }
        }

        // Members
        #region members
        private List<Linea> lineas;
        private Bitmap bitmap;
        private System.Drawing.Pen pen1 = new System.Drawing.Pen(Color.FromArgb(150, 0, 255, 0), 1F);
        private System.Drawing.Pen pen3 = new System.Drawing.Pen(Color.GreenYellow, 5F);
        private System.Drawing.Pen pen2 = new System.Drawing.Pen(Color.Black, 5F);
        private Point origin; 
        private int size = 800;
        private int margin = 150;
        private int angle = 0;
        private bool sentidoAscendente = true;
        #endregion

        #region Constructor
        public Form1()
        {
            InitializeComponent();
        }
        #endregion

        private void Form1_Load(object sender, EventArgs e)
        {
            // Define origin and bitmap
            origin = new Point(size / 2 + margin, size / 2 + margin);
            bitmap = new Bitmap(size + margin + 50, size + margin + 50);
            lineas = new List<Linea>();

            // Initialize the full structure
            for (int i = 0; i <= 180; i++)
            {
                lineas.Add(new Linea()
                {
                    alpha = 0,
                    angle = i,
                    color = Color.GreenYellow,
                    distancia = 255
                });
            }

            // Configure serial port to receive information from Arudino
            // Locate the first COM port ocurrence with Arudino name
            using (var searcher = new ManagementObjectSearcher("SELECT * FROM Win32_PnPEntity WHERE Caption like '%(COM%'"))
            {
                var portnames = SerialPort.GetPortNames();
                var ports = searcher?.Get().Cast<ManagementBaseObject>().ToList().Select(p => p["Caption"].ToString());
                var portList = portnames.Where(n => ports.FirstOrDefault(s => s.Contains(n)).Contains("Arduino"))?.FirstOrDefault();
                if (portList != null)
                {
                    serialPort1.PortName = portList;
                    serialPort1.BaudRate = 9600;
                    serialPort1.Parity = Parity.None;
                    serialPort1.StopBits = StopBits.One;
                    serialPort1.Handshake = Handshake.None;
                    serialPort1.ReadTimeout = 1000;
                    serialPort1.NewLine = "\n";
                    serialPort1.RtsEnable = true;
                    serialPort1.ErrorReceived += serialPort1_ErrorReceived;
                    serialPort1.DataReceived += serialPort1_DataReceived;
                    serialPort1.Open();
                }
            }
        }

        // This method is used to draw a Text within the bitmap area, passing an angle and a length
        private void drawTextAngle(Graphics g, Point origin, float angle, float length, String text = null)
        {
            Point point = new Point((int)(origin.X + Math.Cos(angle * Math.PI / 180) * length), (int)(origin.Y - Math.Sin(angle * Math.PI / 180) * length));
            if (angle > 90) point.X -= 80;
            g.DrawString((text==null)?angle + "º":text, new Font("Tahoma", 20), Brushes.White, point.X, point.Y);
        }

        // This method is used to draw a Line within the bitmap area given an origin, an angle and a length
        private void drawLineAngle(Graphics g, System.Drawing.Pen pen, Point origin, double angle, float length)
        {
            Point p = new Point(origin.X, origin.Y);
            Point point2 = new Point((int)(p.X + Math.Cos(angle * Math.PI / 180) * length), (int)(p.Y - Math.Sin(angle * Math.PI / 180) * length));
            g.DrawLine(pen, p, point2);
        }

        // This method is used to draw am Arc within the bitmap area given an origin and a size
        private void drawArcSize(Graphics g, System.Drawing.Pen pen, Point origin, float size)
        {
            float px = origin.X - (size / 2);
            float py = origin.Y - (size / 2);
            g.DrawArc(pen, px, py, size, size, -180, 180);
        }

        // Method used to draw the scaffolding of the graph, the information and text 
        private void drawBase(Graphics g)
        {
            // Draw the arcs
            drawArcSize(g, pen1, origin, size);
            drawArcSize(g, pen1, origin, size * 0.75f);
            drawArcSize(g, pen1, origin, size * 0.50f);
            drawArcSize(g, pen1, origin, size * 0.25f);

            // Draw the scale lines
            drawLineAngle(g, pen1, origin, 0, 1.05f * (size/2));
            drawLineAngle(g, pen1, origin, 30, 1.05f * (size / 2));
            drawLineAngle(g, pen1, origin, 60, 1.05f * (size / 2));
            drawLineAngle(g, pen1, origin, 90, 1.05f * (size / 2));
            drawLineAngle(g, pen1, origin, 120, 1.05f * (size / 2));
            drawLineAngle(g, pen1, origin, 150, 1.05f * (size / 2));
            drawLineAngle(g, pen1, origin, 180, 1.05f * (size / 2));

            // Draw the text with the information
            drawTextAngle(g, origin, 30, 1.05f * (size / 2));
            drawTextAngle(g, origin, 60, 1.05f * (size / 2));
            drawTextAngle(g, origin, 90, 1.05f * (size / 2));
            drawTextAngle(g, origin, 120, 1.05f * (size / 2));
            drawTextAngle(g, origin, 150, 1.05f * (size / 2));
            drawTextAngle(g, origin, -1, 0.75f * (size / 2), "90cm");
            drawTextAngle(g, origin, -1, 0.50f * (size / 2), "60cm");
            drawTextAngle(g, origin, -1, 0.25f * (size / 2), "30cm");
        }

        // Callback to managed errors coming from the serial port side
        private void serialPort1_ErrorReceived(object sender, SerialErrorReceivedEventArgs e)
        {
            Console.WriteLine("Error");
        }

        // Listener for incoming messages from Arduino serial port connection
        private void serialPort1_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            string valores = serialPort1.ReadExisting();
            string[] parejasDatos = valores.Split('\n');
            foreach (var dato in parejasDatos )
            {
                string[] angulo_distancia = dato.Split('/');
                if (angulo_distancia.Count() == 2)
                {
                    int angulo = 0; 
                    int distancia = 0;

                    int.TryParse(angulo_distancia[0], out angulo);
                    int.TryParse(angulo_distancia[1], out distancia);

                    this.Invoke(new Action(() =>
                    {
                        procesarNuevodato(angulo, distancia);
                    }
                    ));
                }
            }
        }

        // Method to update the information of the graph
        private void procesarNuevodato(int angle, int distancia)
        {
            Graphics g = Graphics.FromImage(bitmap);
            g.Clear(Color.Black);

            foreach (var linea in lineas)
            {
                if (linea.angle == angle)
                {
                    linea.alpha = 255; // Draw the current line full bright
                    linea.distancia = distancia > 120 ? 120 : distancia; 
                    if (distancia < 120) // Change color depending of the distance
                        linea.color = Color.Red;
                    else
                        linea.color = Color.GreenYellow;
                }

                // Decrement the bright of the rest of the lines
                linea.alpha -= 3;
                if (linea.alpha < 0)
                {
                    linea.alpha = 0;
                }
                else
                {
                    Pen penlinea = new Pen(Color.FromArgb(linea.alpha, linea.color.R, linea.color.G, linea.color.B), 5F);
                    drawLineAngle(g, penlinea, origin, linea.angle, linea.distancia * 1.0f * (size / 2) / 120 );
                    penlinea.Dispose();
                }
            }
            drawBase(g);
            g.Dispose();
            pictureBox1.Image = bitmap;
        }

        // Method to close the app
        private void button1_Click(object sender, EventArgs e)
        {
            serialPort1.Close();
            Application.Exit();
        }
    }
}
